package org.expeditee.auth.account;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.cert.CertificateException;
import java.sql.SQLException;
import java.util.Map;
import java.util.Properties;

import javax.crypto.SecretKey;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.PasswordAuthentication;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;

import org.expeditee.auth.AuthenticatorBrowser;
import org.expeditee.auth.tags.AuthenticationTag;
import org.expeditee.gui.DisplayController;
import org.expeditee.gui.FrameIO;
import org.expeditee.gui.MessageBay;

public class Password {
	/*
	 * Changes the recorded password for a user in the key store.
	 */
	public static void changePassword(Map<AuthenticationTag, String> userdata) throws NoSuchAlgorithmException, KeyStoreException, FileNotFoundException, CertificateException, IOException, ClassNotFoundException, SQLException {
		String username = userdata.get(AuthenticationTag.Username);
		String password = userdata.get(AuthenticationTag.Password);
		String newpassword = userdata.get(AuthenticationTag.NewPassword);

		final SecretKey key = AuthenticatorBrowser.getInstance().getSecretKey(username, password);
		if (key == null) {
			MessageBay.errorMessage("The username + existing password combination was incorrect.");
		} else {
			AuthenticatorBrowser.getInstance().putKey(username, newpassword, key);
			MessageBay.displayMessage("Password changed successfully.");
			DisplayController.setCurrentFrame(FrameIO.LoadFrame("multiuser1"), true);
		}
	}

	public static void generateAndDeliverIntergalacticNumber(Map<AuthenticationTag, String> userData) {
		String username = userData.get(AuthenticationTag.Username);
		String email = userData.get(AuthenticationTag.Email);
		try {
			// Generate message text.
			String intergalacticNumber = AuthenticatorBrowser.getInstance().newIntergalacticNumber(username, email);
			String nl = System.getProperty("line.separator");
			StringBuilder sb = new StringBuilder();
			sb.append("You are receiving this email because someone is attempting to reset your Expeditee password." + nl);
			sb.append("If you did not make this request then no action is required." + nl);
			sb.append("If it was you who made this request, the following string of characters is your intergalactic number: " + intergalacticNumber + nl);
			
			// Establish properties for email.
			Properties properties = System.getProperties();
			properties.setProperty("mail.transport.protocol", "smtp");
			properties.setProperty("mail.smtp.host", "smtp.gmail.com");
			properties.setProperty("mail.smtp.port", "465");
			properties.setProperty("mail.smtp.starttls.enable", "true");
			properties.setProperty("mail.smtp.auth", "true");
			properties.setProperty("mail.smtp.debug", "true");
			properties.setProperty("mail.smtp.auth", "true");
			properties.setProperty("mail.smtp.socketFactory.port", "465");
			properties.setProperty("mail.smtp.socketFactory.class", "javax.net.ssl.SSLSocketFactory");
			properties.setProperty("mail.smtp.socketFactory.fallback", "false");
			
			Session session = Session.getDefaultInstance(properties, new javax.mail.Authenticator() {
				@Override
				protected PasswordAuthentication getPasswordAuthentication() {
					return new PasswordAuthentication("noreply.expeditee", "intergalacticnumber");
				};
			});
			
			// construct email message
			final MimeMessage message = new MimeMessage(session);
			message.setFrom(new InternetAddress("noreply@expeditee.com"));
			message.addRecipient(Message.RecipientType.TO, new InternetAddress(email));
			message.setSubject("Expeditee Password Recovery");
			message.setText(sb.toString());
			
			// send email message
			Transport.send(message);		
		} catch (KeyStoreException | NoSuchAlgorithmException | CertificateException | ClassNotFoundException
				| IOException | SQLException | MessagingException e) {
			e.printStackTrace();
		}
	}
	
	public static void confirmIntergalacticNumberAndGenerateSubstituteAccount(Map<AuthenticationTag, String> userData) {
		String username = userData.get(AuthenticationTag.Username);
		String email = userData.get(AuthenticationTag.Email);
		String intergalacticNumber = userData.get(AuthenticationTag.IntergalacticNumber);
		String newPassword = userData.get(AuthenticationTag.NewPassword);
		String newPasswordAgain = userData.get(AuthenticationTag.NewPasswordAgain);
		
		try {
			boolean match = AuthenticatorBrowser.getInstance().confirmIntergalaticNumber(username, email, intergalacticNumber);		
			if (match) {
				MessageBay.displayMessage(":)");
			} else {
				MessageBay.displayMessage(":(");
			}
		} catch (NoSuchAlgorithmException | KeyStoreException | CertificateException | ClassNotFoundException
				| IOException | SQLException e) {
			e.printStackTrace();
		}
	}
}
